import requests
from bs4 import BeautifulSoup

_HEADERS = {
    "User-Agent": (
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_3) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/72.0.3626.121 Safari/537.36"
    ),
}

_CHART_URL = "https://music.bugs.co.kr/chart/track/realtime/total"


def get_songs() -> list[dict[str, str]]:
    res = requests.get(_CHART_URL, headers=_HEADERS)
    res.raise_for_status()
    soup = BeautifulSoup(res.text, "html.parser")
    rows = soup.select("table.byChart tbody tr")

    song_list: list[dict[str, str]] = []

    for row in rows:
        track_id = row.get("trackid")
        title_tag = row.select_one("p.title a")
        artist_tag = row.select_one("p.artist a")
        album_tag = row.select_one("a.album")

        if not (track_id and title_tag and artist_tag and album_tag):
            continue

        song_list.append(
            {
                "track_id": track_id,
                "title": title_tag.text.strip(),
                "artist": artist_tag.text.strip(),
                "album": album_tag.text.strip(),
            }
        )

    return song_list



import pytest
import responses
from requests.exceptions import HTTPError

from melon_top100.bugs import get_songs

SAMPLE_HTML = """
<table class="list trackList byChart">
<tbody>
<tr trackid="130013303" artistid="20067194" albumid="39572764" mvid="637549" multiartist="N" rowtype="track">
  <th scope="row">
    <p class="title">
      <a href="javascript:;" title="Adrenaline">Adrenaline</a>
    </p>
  </th>
  <td class="left">
    <p class="artist">
      <a href="https://music.bugs.co.kr/artist/20067194" title="ATEEZ">ATEEZ</a>
    </p>
  </td>
  <td class="left">
    <a class="album" href="https://music.bugs.co.kr/album/39572764" title="GOLDEN HOUR : Part.4">GOLDEN HOUR : Part.4</a>
  </td>
</tr>
<tr trackid="130013304" artistid="20067195" albumid="39572765" mvid="637550" multiartist="N" rowtype="track">
  <th scope="row">
    <p class="title">
      <a href="javascript:;" title="Second Song">Second Song</a>
    </p>
  </th>
  <td class="left">
    <p class="artist">
      <a href="https://music.bugs.co.kr/artist/20067195" title="Artist B">Artist B</a>
    </p>
  </td>
  <td class="left">
    <a class="album" href="https://music.bugs.co.kr/album/39572765" title="Album B">Album B</a>
  </td>
</tr>
</tbody>
</table>
"""


@responses.activate
def test_get_songs():
    responses.add(
        responses.GET,
        "https://music.bugs.co.kr/chart/track/realtime/total",
        body=SAMPLE_HTML,
        status=200,
    )

    songs = get_songs()

    assert len(songs) == 2
    assert songs[0] == {
        "track_id": "130013303",
        "title": "Adrenaline",
        "artist": "ATEEZ",
        "album": "GOLDEN HOUR : Part.4",
    }
    assert songs[1]["track_id"] == "130013304"


@responses.activate
def test_get_songs_empty():
    empty_html = '<table class="list trackList byChart"><tbody></tbody></table>'
    responses.add(
        responses.GET,
        "https://music.bugs.co.kr/chart/track/realtime/total",
        body=empty_html,
        status=200,
    )

    songs = get_songs()

    assert songs == []


@responses.activate
def test_get_songs_skips_incomplete_row():
    html = """
    <table class="list trackList byChart">
    <tbody>
    <tr trackid="99999" artistid="1" albumid="1" rowtype="track">
      <th scope="row">
        <p class="title">
          <a href="javascript:;" title="Incomplete">Incomplete</a>
        </p>
      </th>
    </tr>
    </tbody>
    </table>
    """
    responses.add(
        responses.GET,
        "https://music.bugs.co.kr/chart/track/realtime/total",
        body=html,
        status=200,
    )

    songs = get_songs()

    assert songs == []


@responses.activate
def test_get_songs_http_error():
    responses.add(
        responses.GET,
        "https://music.bugs.co.kr/chart/track/realtime/total",
        status=500,
    )

    with pytest.raises(HTTPError):
        get_songs()



import pytest
import responses
from requests.exceptions import HTTPError

from melon_top100.melon import get_like_count, get_songs

SAMPLE_HTML = """
<div class="d_song_list">
<table>
<tbody>
<tr data-song-no="12345">
  <td>
    <div class="wrap_song_info">
      <a href="javascript:melon.play.playSong('12345');">첫 번째 노래</a>
      <a href="javascript:melon.link.goAlbumDetail('111');">첫 번째 앨범</a>
      <a href="javascript:melon.link.goArtistDetail('222');">가수A</a>
    </div>
  </td>
</tr>
<tr data-song-no="67890">
  <td>
    <div class="wrap_song_info">
      <a href="javascript:melon.play.playSong('67890');">두 번째 노래</a>
      <a href="javascript:melon.link.goAlbumDetail('333');">두 번째 앨범</a>
      <a href="javascript:melon.link.goArtistDetail('444');">가수B</a>
    </div>
  </td>
</tr>
</tbody>
</table>
</div>
"""

SAMPLE_LIKE_RESPONSE = {
    "contsLike": [
        {"CONTSID": 12345, "SUMMCNT": 5000},
        {"CONTSID": 67890, "SUMMCNT": 3000},
    ]
}


@responses.activate
def test_get_songs():
    responses.add(
        responses.GET,
        "http://www.melon.com/chart/index.htm",
        body=SAMPLE_HTML,
        status=200,
    )

    songs = get_songs()

    assert len(songs) == 2
    assert songs[0] == {
        "song_no": "12345",
        "title": "첫 번째 노래",
        "album": "첫 번째 앨범",
        "artist": "가수A",
    }
    assert songs[1]["song_no"] == "67890"


@responses.activate
def test_get_songs_empty():
    empty_html = '<div class="d_song_list"><table><tbody></tbody></table></div>'
    responses.add(
        responses.GET,
        "http://www.melon.com/chart/index.htm",
        body=empty_html,
        status=200,
    )

    songs = get_songs()

    assert songs == []


@responses.activate
def test_get_songs_skips_incomplete_row():
    html = """
    <div class="d_song_list">
    <table>
    <tbody>
    <tr data-song-no="11111">
      <td>
        <div class="wrap_song_info">
          <a href="javascript:melon.play.playSong('11111');">노래</a>
        </div>
      </td>
    </tr>
    </tbody>
    </table>
    </div>
    """
    responses.add(
        responses.GET,
        "http://www.melon.com/chart/index.htm",
        body=html,
        status=200,
    )

    songs = get_songs()

    assert songs == []


@responses.activate
def test_get_songs_http_error():
    responses.add(
        responses.GET,
        "http://www.melon.com/chart/index.htm",
        status=500,
    )

    with pytest.raises(HTTPError):
        get_songs()


@responses.activate
def test_get_like_count():
    responses.add(
        responses.GET,
        "https://www.melon.com/commonlike/getSongLike.json",
        json=SAMPLE_LIKE_RESPONSE,
        status=200,
    )

    result = get_like_count(12345)

    assert result == {"12345": 5000, "67890": 3000}


@responses.activate
def test_get_like_count_with_list():
    responses.add(
        responses.GET,
        "https://www.melon.com/commonlike/getSongLike.json",
        json=SAMPLE_LIKE_RESPONSE,
        status=200,
    )

    result = get_like_count([12345, 67890])

    assert result == {"12345": 5000, "67890": 3000}
    assert "contsIds=12345%2C67890" in responses.calls[0].request.url


@responses.activate
def test_get_like_count_http_error():
    responses.add(
        responses.GET,
        "https://www.melon.com/commonlike/getSongLike.json",
        status=403,
    )

    with pytest.raises(HTTPError):
        get_like_count(12345)



import requests
from bs4 import BeautifulSoup

_HEADERS = {
    "User-Agent": (
        "Mozilla/5.0 (Macintosh; Intel Mac OS X 10_14_3) "
        "AppleWebKit/537.36 (KHTML, like Gecko) "
        "Chrome/72.0.3626.121 Safari/537.36"
    ),
}


def get_songs() -> list[dict[str, str]]:
    res = requests.get("http://www.melon.com/chart/index.htm", headers=_HEADERS)
    res.raise_for_status()
    soup = BeautifulSoup(res.text, "html.parser")
    tr_tag_list = soup.select(".d_song_list tbody tr")

    song_list: list[dict[str, str]] = []

    for tr_tag in tr_tag_list:
        song_no = tr_tag["data-song-no"]
        song_tag = tr_tag.select_one("a[href*=playSong]")
        album_tag = tr_tag.select_one(".wrap_song_info a[href*=goAlbumDetail]")
        artist_tag = tr_tag.select_one("a[href*=goArtistDetail]")

        if not (song_tag and album_tag and artist_tag):
            continue

        song_list.append(
            {
                "song_no": song_no,
                "title": song_tag.text,
                "album": album_tag.text,
                "artist": artist_tag.text,
            }
        )

    return song_list


def get_like_count(song_no_list: int | list[int]) -> dict[str, int]:
    api_url = "https://www.melon.com/commonlike/getSongLike.json"
    if isinstance(song_no_list, list):
        conts_ids = ",".join(str(n) for n in song_no_list)
    else:
        conts_ids = str(song_no_list)
    params = {"contsIds": conts_ids}
    res = requests.get(api_url, params=params, headers=_HEADERS)
    res.raise_for_status()
    response = res.json()
    like_list: list[dict] = response["contsLike"]
    return {str(song["CONTSID"]): song["SUMMCNT"] for song in like_list}
